@@ -1,4 +1,4 @@
-/* XLanguage - the eXtensible Language
+/* X Language - the eXtensible Language
  * Copyright (C) 2001 Patrick Deschenes
  *
  * This program is free software; you can redistribute it and/or
@@ -21,11 +21,17 @@
 
 #include <config.h>
 
+#include <xldefs.h>
 #include <xlbuiltin.h>
-#include <xllang.h>
 #include <xlclass.h>
-#include <xlmain.h>
+#include <xlfield.h>
 #include <xlmeth.h>
+#include <xlfnct.h>
+#include <xlexpr.h>
+#include <xldata.h>
+#include <xltype.h>
+#include <xlmain.h>
+#include <xllang.h>
 
 XLClass* g_xl_builtin_current_class = NULL;
 
@@ -38,23 +44,17 @@ xl_builtin_init ()
   /* Declarators
    */
 
-  xl_main_add_fnct_builtin (g_xl_main_global, "fnct", xl_builtin_fnct);
-  xl_main_add_fnct_builtin (g_xl_main_global, "dlfnct", xl_builtin_dlfnct);
-  xl_main_add_fnct_builtin (g_xl_main_global, "global", xl_builtin_global);
-  xl_main_add_fnct_builtin (g_xl_main_global, "const", xl_builtin_const);
-  xl_main_add_fnct_builtin (g_xl_main_global, "local", xl_builtin_local);
-  xl_main_add_fnct_builtin (g_xl_main_global, "class", xl_builtin_class); 
-  xl_main_add_fnct_builtin (g_xl_main_global, "native_class", xl_builtin_native_class);
-  xl_main_add_fnct_builtin (g_xl_main_global, "native_field", xl_builtin_native_field);
-  xl_main_add_fnct_builtin (g_xl_main_global, "native_method", xl_builtin_native_method);
-  xl_main_add_fnct_builtin (g_xl_main_global, "meth", xl_builtin_meth);
-  xl_main_add_fnct_builtin (g_xl_main_global, "meth_static", xl_builtin_meth_static);
-  xl_main_add_fnct_builtin (g_xl_main_global, "prop", xl_builtin_prop);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@fnct", xl_builtin_fnct);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@dlfnct", xl_builtin_dlfnct);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@global", xl_builtin_global);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@const", xl_builtin_const);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@local", xl_builtin_local);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@class", xl_builtin_class); 
+  xl_main_add_fnct_builtin (g_xl_main_global, "@meth", xl_builtin_meth);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@meth_static", xl_builtin_meth_static);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@prop", xl_builtin_prop);
 
-  xl_main_add_fnct_builtin (g_xl_main_global, "cast", xl_builtin_cast);
-  xl_main_add_fnct_builtin (g_xl_main_global, "addr_of", xl_builtin_addr_of);
-  xl_main_add_fnct_builtin (g_xl_main_global, "set_raw", xl_builtin_set_raw);
-  xl_main_add_fnct_builtin (g_xl_main_global, "str_setsize", xl_builtin_str_setsize);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@cast", xl_builtin_cast);
 
   xl_main_add_fnct_builtin (g_xl_main_global, "|", xl_builtin_binary_or);
   xl_main_add_fnct_builtin (g_xl_main_global, "&", xl_builtin_binary_and);
@@ -64,20 +64,22 @@ xl_builtin_init ()
   xl_main_add_fnct_builtin (g_xl_main_global, "*", xl_builtin_mul);
   xl_main_add_fnct_builtin (g_xl_main_global, "/", xl_builtin_div);
 
-  xl_main_add_fnct_builtin (g_xl_main_global, "import", xl_builtin_import);
-  xl_main_add_fnct_builtin (g_xl_main_global, "ifdef", xl_builtin_ifdef);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@import", xl_builtin_import);
 
-  xl_main_add_fnct_builtin (g_xl_main_global, "new", xl_builtin_new);  
+  xl_main_add_fnct_builtin (g_xl_main_global, "@new", xl_builtin_new);  
   xl_main_add_fnct_builtin (g_xl_main_global, ".", xl_builtin_access);  
   xl_main_add_fnct_builtin (g_xl_main_global, "::", xl_builtin_invoke);
   xl_main_add_fnct_builtin (g_xl_main_global, "[]", xl_builtin_array);
-  xl_main_add_fnct_builtin (g_xl_main_global, "arr_getsize", xl_builtin_array_getsize);
-  xl_main_add_fnct_builtin (g_xl_main_global, "arr_resize", xl_builtin_array_resize);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@array_add", xl_builtin_array_add);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@array_get_size", xl_builtin_array_get_size);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@array_set_size", xl_builtin_array_set_size);
 
-  xl_main_add_fnct_builtin (g_xl_main_global, "return", xl_builtin_return);
-  xl_main_add_fnct_builtin (g_xl_main_global, "if", xl_builtin_if);
-  xl_main_add_fnct_builtin (g_xl_main_global, "while", xl_builtin_while);
-  xl_main_add_fnct_builtin (g_xl_main_global, "for", xl_builtin_for);
+
+  xl_main_add_fnct_builtin (g_xl_main_global, "@return", xl_builtin_return);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@if", xl_builtin_if);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@while", xl_builtin_while);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@for", xl_builtin_for);
+  xl_main_add_fnct_builtin (g_xl_main_global, "@foreach", xl_builtin_foreach);
 
   xl_main_add_fnct_builtin (g_xl_main_global, "<", xl_builtin_cmp_l);
   xl_main_add_fnct_builtin (g_xl_main_global, ">", xl_builtin_cmp_g);
@@ -142,10 +144,20 @@ xl_builtin_fnct (XLExpr* p_expr)
       return;
     }  
 
-  l_code = xl_expr_get (p_expr, l_size - 1);
-  l_fnct = xl_main_add_fnct_expr (g_xl_main_global, x_string_get_str (l_function_name), l_code);
-  x_unref (l_function_name);
-  x_unref (l_code);
+  if ((l_size % 2) != 0)
+    {
+      l_code = xl_expr_get (p_expr, l_size - 1);
+      l_fnct = xl_main_add_fnct_expr (g_xl_main_global, x_string_get_str (l_function_name), l_code);
+      x_unref (l_function_name);
+      x_unref (l_code);
+    }
+  else
+    {
+      l_fnct = xl_main_add_fnct_proto (g_xl_main_global, x_string_get_str (l_function_name));
+      x_unref (l_function_name);
+
+      return;
+    }
 
   for (i = 0; i < ((l_size - 3) / 2); i++)
     {
@@ -401,7 +413,7 @@ xl_builtin_class (XLExpr* p_expr)
 
   l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 0));
   l_text = x_addref (XString, l_expr->identifier);
-  l_class = xl_class_new (l_text, XL_CLASS_DEFAULT, 0);
+  l_class = xl_class_new (l_text, 0);
   x_unref (l_text);
   x_unref (l_expr);
 
@@ -410,7 +422,11 @@ xl_builtin_class (XLExpr* p_expr)
 
   l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 1));
   if (l_expr->expr_type == XL_EXPR_TYPE_IDENTIFIER)
-    i = 2;
+    {
+      x_unref (l_class->parent_class_name);
+      l_class->parent_class_name = x_addref (XString, l_expr->identifier);
+      i = 2;
+    }
   else
     i = 1;
 
@@ -428,78 +444,6 @@ xl_builtin_class (XLExpr* p_expr)
 
 }
 
-xvoid
-xl_builtin_native_class (XLExpr* p_expr)
-{
-  XLExpr*  l_expr;
-  XString* l_text;
-  XLClass* l_class;
-  xuint     l_size, i;
-
-  l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 0));
-  l_text = x_addref (XString, l_expr->identifier);
-  l_class = xl_class_new (l_text, XL_CLASS_BINDING_PTR, 0);
-  x_unref (l_text);
-  x_unref (l_expr);
-
-  g_xl_builtin_current_class = l_class;
-  l_size = x_list_get_size (p_expr->list_expr);
-
-  l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 1));
-  if (l_expr->expr_type == XL_EXPR_TYPE_IDENTIFIER)
-    i = 2;
-  else
-    i = 1;
-
-  x_unref (l_expr);
-
-  for (; i<l_size; i++)
-    {
-      l_expr = xl_expr_get (p_expr, i);
-      xl_expr_evaluate (l_expr);
-      x_unref (l_expr);
-    }
-  
-  xl_main_add_class (g_xl_main_global, l_class);
-  x_unref (l_class);
-
-}
-
-xvoid
-xl_builtin_native_field (XLExpr* p_expr)
-{
-  XLType*  l_type;
-  XString* l_name;
-  XLField* l_field;
-  XLExpr*  l_expr;
-  xuint     l_offset;
-
-  l_expr = xl_expr_get (p_expr, 2);
-  xl_expr_evaluate (l_expr);
-  l_offset = x_access (xl_data_get_raw (l_expr->data), 0, xuint);
-  x_unref (l_expr);
-
-  l_expr = xl_expr_get (p_expr, 0);
-  l_type = x_addref (XLType, l_expr->type);
-  x_unref (l_expr);
-
-  l_expr = xl_expr_get (p_expr, 1);
-  l_name = x_addref (XString, l_expr->identifier);
-  x_unref (l_expr);
-
-  l_field = xl_field_new (l_name, l_type);
-  x_unref (l_name);
-  x_unref (l_type);
-
-  xl_class_add_field_offset (g_xl_builtin_current_class, l_field, l_offset);
-  x_unref (l_field);
-}
-
-xvoid
-xl_builtin_native_method (XLExpr* p_expr)
-{
-}
-
 xvoid
 xl_builtin_prop (XLExpr* p_expr)
 {
@@ -507,7 +451,7 @@ xl_builtin_prop (XLExpr* p_expr)
   XLType*  l_prop_type;
   XString* l_prop_name;
   XLField* l_field;
-  xuint     l_size;
+  xuint    l_size;
 
 
   l_size = xl_expr_get_size (p_expr);
@@ -553,11 +497,12 @@ xl_builtin_meth (XLExpr* p_expr)
   XLExpr*  l_subexpr;
   XString* l_name;
   XString* l_fnctname;
+  XLType*  l_type;
   XLFnct*  l_fnct;
   XLField* l_field;
   XLMeth*  l_meth;
   xuint	   l_size;
-  xuint     i;
+  xuint    i;
 
   l_size = x_list_get_size (p_expr->list_expr);
   
@@ -591,18 +536,27 @@ xl_builtin_meth (XLExpr* p_expr)
    */
   l_name = x_string_new ();
   x_string_set_str (l_name, "self");
-  l_field = xl_field_new (l_name, NULL);
+
+  l_type = xl_type_new (XL_TYPE_TYPE_UNKNOWN, XL_TYPE_ARRAY_TYPE_NONE, 0, g_xl_builtin_current_class->name);
+  l_field = xl_field_new (l_name, l_type);
+  x_unref (l_type);
+
   xl_fnct_add_param (l_fnct, l_field);
   x_unref (l_name);
   x_unref (l_field);
 
   for (i=0; i<(l_size / 2) - 1; i++)
     {
+      l_subexpr = XL_EXPR (x_list_get (p_expr->list_expr, (i * 2) + 2));
+      l_type = x_addref (XLType, l_subexpr->type);
+      x_unref (l_type);
+
       l_subexpr = XL_EXPR (x_list_get (p_expr->list_expr, (i * 2) + 3));
 
-      l_field = xl_field_new (l_subexpr->identifier, NULL);
+      l_field = xl_field_new (l_subexpr->identifier, l_type);
       xl_fnct_add_param (l_fnct, l_field);
 
+      x_unref (l_type);
       x_unref (l_field);
       x_unref (l_subexpr);
     }
@@ -615,7 +569,9 @@ xl_builtin_meth_static (XLExpr* p_expr)
   XLExpr*  l_subexpr;
   XString* l_name;
   XLFnct*  l_fnct;
+  XLType*  l_type;
   XLField* l_field;
+  XLMeth*  l_meth;
   xuint	   l_size;
   xuint    i;
 
@@ -628,6 +584,9 @@ xl_builtin_meth_static (XLExpr* p_expr)
   x_string_add (l_name, l_expr->identifier);
   x_unref (l_expr);
 
+  l_meth = xl_meth_new (l_expr->identifier, l_name, XL_METH_STATIC);
+  xl_class_add_method (g_xl_builtin_current_class, l_meth);
+
   l_expr = XL_EXPR (x_list_get (p_expr->list_expr, l_size - 1));
   l_fnct = xl_main_add_fnct_expr (g_xl_main_global, x_string_get_str (l_name), l_expr);
   x_unref (l_name);
@@ -635,11 +594,16 @@ xl_builtin_meth_static (XLExpr* p_expr)
 
   for (i=0; i<(l_size / 2) - 1; i++)
     {
+      l_subexpr = XL_EXPR (x_list_get (p_expr->list_expr, (i * 2) + 2));
+      l_type = x_addref (XLType, l_subexpr->type);
+      x_unref (l_type);
+
       l_subexpr = XL_EXPR (x_list_get (p_expr->list_expr, (i * 2) + 3));
 
-      l_field = xl_field_new (l_subexpr->identifier, NULL);
+      l_field = xl_field_new (l_subexpr->identifier, l_type);   
       xl_fnct_add_param (l_fnct, l_field);
 
+      x_unref (l_type);
       x_unref (l_field);
       x_unref (l_subexpr);
     }
@@ -684,14 +648,30 @@ xl_builtin_invoke (XLExpr* p_expr)
   l_data = x_addref (XLData, l_expr->data);
   if (l_data)
     {
-      l_class = x_addref (XLClass, l_data->type->class);
+      if (l_data->type->type != XL_TYPE_TYPE_OBJECT)
+	{
+	  printf ("error: not an object\n");
+	  exit (-1);
+	}
+
+      l_class = xl_data_get_class (l_data);
       x_unref (l_data);
+      if (!l_class)
+	{
+	  printf ("error-invoke: object is null\n");
+	  exit (-1); 
+	}
     }
   else
-    l_class = xl_main_get_class (g_xl_main_global, l_expr->identifier);
+    {      
+      l_class = xl_main_get_class (g_xl_main_global, l_expr->identifier);
+    }
 
   x_unref (l_expr);
 
+  if (!l_class)
+    return;
+
   l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 1));
   l_meth = xl_class_get_method (l_class, l_expr->identifier);
   x_unref (l_class);
@@ -700,7 +680,8 @@ xl_builtin_invoke (XLExpr* p_expr)
     {
       printf ("error: method '%s' from class '%s' doesn't exists\n",
 	      x_string_get_str (l_expr->identifier),
-	      x_string_get_str (l_class->name));
+	      x_string_get_str (l_class->name)
+	      );
       
       exit (-1);
     }
@@ -765,7 +746,35 @@ xl_builtin_array (XLExpr* p_expr)
 }
 
 xvoid
-xl_builtin_array_getsize (XLExpr* p_expr)
+xl_builtin_array_add (XLExpr* p_expr)
+{
+  XLExpr* l_expr;
+  XLData* l_data;
+  XLData* l_dst;
+  XLData* l_src;
+  xuint   l_size;
+
+  l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 0));
+  xl_expr_evaluate (l_expr);
+  l_data = l_expr->data;
+  x_unref (l_expr);
+
+  l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 1));
+  xl_expr_evaluate (l_expr);
+  l_src = l_expr->data;
+  x_unref (l_expr);
+
+  l_size = xl_data_array_get_size (l_data);
+  xl_data_array_set_size (l_data, l_size + 1);
+
+  l_dst = xl_data_get_index (l_data, l_size);
+  xl_data_assign (l_dst, l_src);
+
+  x_unref (l_dst);
+}
+
+xvoid
+xl_builtin_array_get_size (XLExpr* p_expr)
 {
   XLExpr* l_expr;
   XLData* l_data;
@@ -777,7 +786,7 @@ xl_builtin_array_getsize (XLExpr* p_expr)
   l_data = x_addref (XLData, l_expr->data);
   x_unref (l_expr);
 
-  l_rettype = xl_type_new (XL_TYPE_TYPE_U32, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
+  l_rettype = xl_type_new (XL_TYPE_TYPE_UINT, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
   l_retdata = xl_data_new (l_rettype, NULL);
   x_unref (l_rettype);
 
@@ -789,7 +798,7 @@ xl_builtin_array_getsize (XLExpr* p_expr)
 }
 
 xvoid
-xl_builtin_array_resize (XLExpr* p_expr)
+xl_builtin_array_set_size (XLExpr* p_expr)
 {
   XLExpr* l_expr;
   XLData* l_data;
@@ -813,7 +822,23 @@ xl_builtin_array_resize (XLExpr* p_expr)
 xvoid
 xl_builtin_new (XLExpr* p_expr)
 {
-  printf ("new\n");
+  XLClass*        l_class;
+  XLExpr*         l_expr;
+
+  l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 1));
+  xl_expr_evaluate (l_expr);
+  l_class = xl_type_get_class (l_expr->type);
+  x_unref (l_expr);
+
+  l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 0));  
+  xl_expr_evaluate (l_expr);
+
+  if (l_class)
+    xl_data_object_new (l_expr->data, l_class);
+  else
+    printf ("ERROR: unknown class\n");
+
+  x_unref (l_expr);
 }
 
 xvoid
@@ -822,12 +847,18 @@ xl_builtin_assign  (XLExpr* p_expr)
   XLExpr* l_expr;
   XLData* l_src;
   XLData* l_dst;
+  XLData* l_data;
 
   l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 0));
   xl_expr_evaluate (l_expr);
   l_dst = x_addref (XLData, l_expr->data);
   x_unref (l_expr);
 
+  x_unref (p_expr->data);
+  l_data = xl_data_new (l_dst->type, NULL);
+  xl_data_assign (l_data, l_dst);
+  p_expr->data = l_data;
+
   l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 1));  
   xl_expr_evaluate (l_expr);
   l_src = x_addref (XLData, l_expr->data);
@@ -863,86 +894,31 @@ xl_builtin_cast  (XLExpr* p_expr)
   x_unref (l_src);
 }
 
-xvoid
-xl_builtin_addr_of (XLExpr* p_expr)
-{
-  XLExpr* l_expr;
-  XLType* l_type;
-  XLData* l_data;
-
-  l_type = xl_type_new (XL_TYPE_TYPE_POINTER, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
-  l_data = xl_data_new (l_type, NULL);
-  x_unref (l_type);
-
-  l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 0));
-  xl_expr_evaluate (l_expr);
-
-  if (l_expr->data->type->type == XL_TYPE_TYPE_STRING)
-    x_access (xl_data_get_raw (l_data), 0, xptr) = x_access (xl_data_get_raw (l_expr->data), 4, xptr);
-  else
-    x_access (xl_data_get_raw (l_data), 0, xptr) = (xptr) xl_data_get_raw (l_expr->data);
-
-  x_unref (l_expr);
-
-  x_unref (p_expr->data);
-  p_expr->data = l_data;
-}
-
-xvoid
-xl_builtin_set_raw (XLExpr* p_expr)
-{
-  XLExpr* l_expr;
-  XLData* l_data;
-  xptr    l_raw;
-
-  l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 1));
-  xl_expr_evaluate (l_expr);
-  l_raw = x_access (xl_data_get_raw (l_expr->data), 0, xptr);
-  x_unref (l_expr);
-
-  l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 0));
-  xl_expr_evaluate (l_expr);
-  xl_data_set_raw (l_expr->data, l_raw);
-  x_unref (l_expr);
-}
-
-xvoid
-xl_builtin_str_setsize (XLExpr* p_expr)
-{
-  XLExpr*  l_expr;
-  XLData*  l_data;
-  xuint    l_size;
-  xstr     l_str;
-
-  l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 0));
-  xl_expr_evaluate (l_expr);
-  l_data = l_expr->data;
-  x_unref (l_expr);
- 
-  l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 1));
-  xl_expr_evaluate (l_expr);
-  l_size = x_access (xl_data_get_raw (l_expr->data), 0, xuint);
-  x_unref (l_expr);
-
-  l_str = x_resize (x_access (l_data->raw, 4, xstr), l_size + 1);
-  x_access (xl_data_get_raw (l_data), 4, xstr) = l_str;
- 
-  l_str[l_size] = '\0';
-}
-
 xvoid
 xl_builtin_return (XLExpr* p_expr)
 {
   XLExpr* l_expr;
+  XLData* l_data;
   
   l_expr = XL_EXPR (x_list_get (p_expr->list_expr, 0));
 
   if (l_expr)
     {
       xl_expr_evaluate (l_expr);
-      g_xl_fnct_return = x_addref (XLData, l_expr->data);
+
+      l_data = xl_data_new (l_expr->data->type, NULL);
+      xl_data_assign (l_data, l_expr->data);
+
+      x_unref (g_xl_fnct_return);
+      g_xl_fnct_return = l_data;
+
       x_unref (l_expr);
     }
+  else
+    {
+      x_unref (g_xl_fnct_return);
+      g_xl_fnct_return = NULL;
+    }
 
   g_xl_fnct_halt = TRUE;
 }
@@ -1023,6 +999,37 @@ xl_builtin_for (XLExpr* p_expr)
   x_unref (l_code);
 }
 
+xvoid
+xl_builtin_foreach (XLExpr* p_expr)
+{
+  XLExpr* l_item;
+  XLExpr* l_array;
+  XLExpr* l_code;
+  XLData* l_dataitem;
+  xuint   i, size;
+  
+  l_item = XL_EXPR (x_list_get (p_expr->list_expr, 0));
+  l_array = XL_EXPR (x_list_get (p_expr->list_expr, 1));
+  l_code = XL_EXPR (x_list_get (p_expr->list_expr, 2));
+
+  xl_expr_evaluate (l_item);
+  xl_expr_evaluate (l_array);
+
+  size = xl_data_array_get_size (l_array->data);
+  for (i=0; i<size; i++)
+    {
+      l_dataitem = xl_data_get_index (l_array->data, i);
+      xl_data_assign (l_item->data, l_dataitem);
+      x_unref (l_dataitem);
+
+      xl_expr_evaluate (l_code);
+    }
+
+  x_unref (l_code);
+  x_unref (l_array);
+  x_unref (l_item);
+}
+
 xvoid
 xl_builtin_inc (XLExpr* p_expr)
 {
@@ -1033,6 +1040,9 @@ xl_builtin_inc (XLExpr* p_expr)
   x_unref (l_expr);
 
   x_access (xl_data_get_raw (l_expr->data), 0, xuint) = x_access (xl_data_get_raw (l_expr->data), 0, xuint) + 1;
+
+  x_unref (p_expr->data);
+  p_expr->data = x_addref (XLData, l_expr->data);
 }
 
 xvoid
@@ -1045,6 +1055,9 @@ xl_builtin_dec (XLExpr* p_expr)
   x_unref (l_expr);
 
   x_access (xl_data_get_raw (l_expr->data), 0, xuint) = x_access (xl_data_get_raw (l_expr->data), 0, xuint) - 1;
+
+  x_unref (p_expr->data);
+  p_expr->data = x_addref (XLData, l_expr->data);
 }
 
 xvoid
@@ -1131,6 +1144,7 @@ xl_builtin_cmp_le (XLExpr* p_expr)
 
   x_unref (l_left);
   x_unref (l_right);
+
   x_unref (p_expr->data);
   p_expr->data = l_data;
 }
@@ -1306,7 +1320,7 @@ xl_builtin_io_write (XLExpr* p_expr)
       x_unref (l_text);
     }
 
-  if (l_data->type->type == XL_TYPE_TYPE_U32)
+  if (l_data->type->type == XL_TYPE_TYPE_UINT)
     {
       printf ("%d", x_access (xl_data_get_raw (l_data), 0, xuint));
     }
@@ -1326,6 +1340,9 @@ xl_builtin_block (XLExpr* p_expr)
       l_expr = XL_EXPR (x_list_get (p_expr->list_expr, i));
       xl_expr_evaluate (l_expr);
       x_unref (l_expr);
+
+      if (g_xl_fnct_halt)
+	return;
     }
 }
 
@@ -1343,9 +1360,9 @@ xl_builtin_binary_or (XLExpr* p_expr)
   xl_expr_evaluate (l_left);
   xl_expr_evaluate (l_right);
 
-  if (l_left->data->type->type == XL_TYPE_TYPE_U32)
+  if (l_left->data->type->type == XL_TYPE_TYPE_UINT)
     {
-      l_type = xl_type_new (XL_TYPE_TYPE_U32, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
+      l_type = xl_type_new (XL_TYPE_TYPE_UINT, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
       l_data = xl_data_new (l_type, NULL);
       x_unref (l_type);
       
@@ -1374,9 +1391,9 @@ xl_builtin_binary_and (XLExpr* p_expr)
   xl_expr_evaluate (l_left);
   xl_expr_evaluate (l_right);
 
-  if (l_left->data->type->type == XL_TYPE_TYPE_U32)
+  if (l_left->data->type->type == XL_TYPE_TYPE_UINT)
     {
-      l_type = xl_type_new (XL_TYPE_TYPE_U32, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
+      l_type = xl_type_new (XL_TYPE_TYPE_UINT, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
       l_data = xl_data_new (l_type, NULL);
       x_unref (l_type);
       
@@ -1399,50 +1416,74 @@ xl_builtin_add (XLExpr* p_expr)
   XLType*  l_type;
   XLData*  l_data;
   XString* l_string;
+  XLData*  l_data_left;
+  XLData*  l_data_right;
+
+  l_data = NULL;
 
   l_left = XL_EXPR (x_list_get (p_expr->list_expr, 0));
   l_right = XL_EXPR (x_list_get (p_expr->list_expr, 1));
 
   xl_expr_evaluate (l_left);
-  xl_expr_evaluate (l_right);
+  l_data_left = xl_data_copy (l_left->data);
 
-  if (l_left->data->type->type == XL_TYPE_TYPE_U32)
+  xl_expr_evaluate (l_right);
+  l_data_right = xl_data_copy (l_right->data);
+
+  if (l_data_left->type->type == XL_TYPE_TYPE_UINT)
     {
-      l_type = xl_type_new (XL_TYPE_TYPE_U32, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
+      l_type = xl_type_new (XL_TYPE_TYPE_UINT, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
       l_data = xl_data_new (l_type, NULL);
       x_unref (l_type);
-      
+
       x_access (xl_data_get_raw (l_data), 0, xuint) = 
-	x_access (xl_data_get_raw (l_left->data), 0, xuint) + 
-	x_access (xl_data_get_raw (l_right->data), 0, xuint);
+	x_access (xl_data_get_raw (l_data_left), 0, xuint) + 
+	x_access (xl_data_get_raw (l_data_right), 0, xuint);
+     }
+
+  if (l_data_left->type->type == XL_TYPE_TYPE_INT)
+    {
+      l_type = xl_type_new (XL_TYPE_TYPE_INT, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
+      l_data = xl_data_new (l_type, NULL);
+      x_unref (l_type);
+
+      x_access (xl_data_get_raw (l_data), 0, xint) = 
+	x_access (xl_data_get_raw (l_data_left), 0, xint) + 
+	x_access (xl_data_get_raw (l_data_right), 0, xint);
+
+      printf ("add int + int\n");
     }
 
-  if (l_left->data->type->type == XL_TYPE_TYPE_F32)
+  if (l_data_left->type->type == XL_TYPE_TYPE_FLOAT)
     {
-      l_type = xl_type_new (XL_TYPE_TYPE_F32, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
+      l_type = xl_type_new (XL_TYPE_TYPE_FLOAT, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
       l_data = xl_data_new (l_type, NULL);
       x_unref (l_type);
 
       x_access (xl_data_get_raw (l_data), 0, xfloat) = 
-	x_access (xl_data_get_raw (l_left->data), 0, xfloat) + 
-	x_access (xl_data_get_raw (l_right->data), 0, xfloat);
+	x_access (xl_data_get_raw (l_data_left), 0, xfloat) + 
+	x_access (xl_data_get_raw (l_data_right), 0, xfloat);
     }
 
-  if (l_left->data->type->type == XL_TYPE_TYPE_STRING)
+  if (l_data_left->type->type == XL_TYPE_TYPE_STRING)
     {
       l_type = xl_type_new (XL_TYPE_TYPE_STRING, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
       l_data = xl_data_new (l_type, NULL);
       x_unref (l_type);
 
       l_string = x_string_new ();
-      x_string_set (l_string, xl_data_get_string (l_left->data));
-      x_string_add (l_string, xl_data_get_string (l_right->data));
+      x_string_set (l_string, xl_data_get_string (l_data_left));
+      x_string_add (l_string, xl_data_get_string (l_data_right));
 
       xl_data_set_string (l_data, l_string);
     }
 
+  x_unref (l_data_left);
+  x_unref (l_data_right);
+
   x_unref (l_left);
   x_unref (l_right);
+
   x_unref (p_expr->data);
   p_expr->data = l_data;
 }
@@ -1461,9 +1502,9 @@ xl_builtin_sub (XLExpr* p_expr)
   xl_expr_evaluate (l_left);
   xl_expr_evaluate (l_right);
 
-  if (l_left->data->type->type == XL_TYPE_TYPE_U32)
+  if (l_left->data->type->type == XL_TYPE_TYPE_UINT)
     {
-      l_type = xl_type_new (XL_TYPE_TYPE_U32, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
+      l_type = xl_type_new (XL_TYPE_TYPE_UINT, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
       l_data = xl_data_new (l_type, NULL);
       x_unref (l_type);
       
@@ -1472,10 +1513,22 @@ xl_builtin_sub (XLExpr* p_expr)
 	x_access (xl_data_get_raw (l_right->data), 0, xuint);
     }
 
-  if (l_left->data->type->type == XL_TYPE_TYPE_F32)
+
+  if (l_left->data->type->type == XL_TYPE_TYPE_INT)
+    {
+      l_type = xl_type_new (XL_TYPE_TYPE_INT, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
+      l_data = xl_data_new (l_type, NULL);
+      x_unref (l_type);
+      
+      x_access (xl_data_get_raw (l_data), 0, xint) = 
+	x_access (xl_data_get_raw (l_left->data), 0, xint) - 
+	x_access (xl_data_get_raw (l_right->data), 0, xint);
+    }
+
+  if (l_left->data->type->type == XL_TYPE_TYPE_FLOAT)
     {
 
-      l_type = xl_type_new (XL_TYPE_TYPE_F32, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
+      l_type = xl_type_new (XL_TYPE_TYPE_FLOAT, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
       l_data = xl_data_new (l_type, NULL);
       x_unref (l_type);
 
@@ -1504,9 +1557,9 @@ xl_builtin_mul (XLExpr* p_expr)
   xl_expr_evaluate (l_left);
   xl_expr_evaluate (l_right);
 
-  if (l_left->data->type->type == XL_TYPE_TYPE_U32)
+  if (l_left->data->type->type == XL_TYPE_TYPE_UINT)
     {
-      l_type = xl_type_new (XL_TYPE_TYPE_U32, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
+      l_type = xl_type_new (XL_TYPE_TYPE_UINT, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
       l_data = xl_data_new (l_type, NULL);
       x_unref (l_type);
       
@@ -1515,10 +1568,10 @@ xl_builtin_mul (XLExpr* p_expr)
 	x_access (xl_data_get_raw (l_right->data), 0, xuint);
     }
 
-  if (l_left->data->type->type == XL_TYPE_TYPE_F32)
+  if (l_left->data->type->type == XL_TYPE_TYPE_FLOAT)
     {
 
-      l_type = xl_type_new (XL_TYPE_TYPE_F32, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
+      l_type = xl_type_new (XL_TYPE_TYPE_FLOAT, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
       l_data = xl_data_new (l_type, NULL);
       x_unref (l_type);
 
@@ -1547,9 +1600,9 @@ xl_builtin_div (XLExpr* p_expr)
   xl_expr_evaluate (l_left);
   xl_expr_evaluate (l_right);
 
-  if (l_left->data->type->type == XL_TYPE_TYPE_U32)
+  if (l_left->data->type->type == XL_TYPE_TYPE_UINT)
     {
-      l_type = xl_type_new (XL_TYPE_TYPE_U32, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
+      l_type = xl_type_new (XL_TYPE_TYPE_UINT, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
       l_data = xl_data_new (l_type, NULL);
       x_unref (l_type);
       
@@ -1558,10 +1611,10 @@ xl_builtin_div (XLExpr* p_expr)
 	x_access (xl_data_get_raw (l_right->data), 0, xuint);
     }
 
-  if (l_left->data->type->type == XL_TYPE_TYPE_F32)
+  if (l_left->data->type->type == XL_TYPE_TYPE_FLOAT)
     {
 
-      l_type = xl_type_new (XL_TYPE_TYPE_F32, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
+      l_type = xl_type_new (XL_TYPE_TYPE_FLOAT, XL_TYPE_ARRAY_TYPE_NONE, 0, NULL);
       l_data = xl_data_new (l_type, NULL);
       x_unref (l_type);
 
@@ -1607,34 +1660,4 @@ xl_builtin_import (XLExpr* p_expr)
     }
 }
 
-xvoid
-xl_builtin_ifdef (XLExpr* p_expr)
-{
-  XLExpr*  l_expr;
-  XString* l_define;
-
-  l_expr = xl_expr_get (p_expr, 0);
-  xl_expr_evaluate (l_expr);
-  l_define = xl_data_get_string (l_expr->data);
-  x_unref (l_expr);
-
-  l_expr = xl_expr_get (p_expr, 1);
-
-  if (x_string_cmp_str (l_define, "OS_WIN32") == 0)
-    {
-#ifdef USE_WIN32
-      xl_expr_evaluate (l_expr);
-#endif
-    }
-
-  if (x_string_cmp_str (l_define, "OS_LINUX") == 0)
-    {
-#ifndef USE_WIN32
-      xl_expr_evaluate (l_expr);
-#endif
-    }
-
-  x_unref (l_define);
-  x_unref (l_expr);
-}
 
